@@ -26,6 +26,7 @@
 #include "gtr-header.h"
 #include "gtr-prefs-manager.h"
 #include "gtr-profile.h"
+#include "gtr-profile-manager.h"
 #include "gtr-utils.h"
 
 #include <glib.h>
@@ -38,14 +39,17 @@
                                          GtrHeaderPrivate))
 
 G_DEFINE_TYPE (GtrHeader, gtr_header, GTR_TYPE_MSG)
-     struct _GtrHeaderPrivate
-     {
-       gint nplurals;
-     };
 
-     static void
-       gtr_header_set_field (GtrHeader * header,
-                             const gchar * field, const gchar * data)
+struct _GtrHeaderPrivate
+{
+  GtrProfileManager *prof_manager;
+  GtrProfile *profile;
+  gint nplurals;
+};
+
+static void
+gtr_header_set_field (GtrHeader * header,
+                      const gchar * field, const gchar * data)
 {
   gchar *msgstr;
 
@@ -70,13 +74,22 @@ parse_nplurals (GtrHeader * header)
 
   if (gtr_prefs_manager_get_use_profile_values () || !plural_forms)
     {
-      const gchar *plural_form;
+      const gchar *plural_form = NULL;
       GtrProfile *profile;
 
-      profile = gtr_application_get_active_profile (GTR_APP);
+      if (header->priv->profile != NULL)
+        profile = header->priv->profile;
+      else
+        {
+          GtrProfileManager *prof_manager;
+
+          prof_manager = gtr_profile_manager_get_default ();
+          profile = gtr_profile_manager_get_active_profile (prof_manager);
+          g_object_unref (prof_manager);
+        }
 
       if (profile)
-        plural_form = gtr_profile_get_plurals (profile);
+        plural_form = gtr_profile_get_plural_forms (profile);
       else if (!plural_forms)
         return;
 
@@ -116,18 +129,47 @@ parse_nplurals (GtrHeader * header)
   g_free (plural_forms);
 }
 
+static void
+profile_removed_cb (GtrProfileManager *prof_manager,
+                    GtrProfile        *profile,
+                    GtrHeader         *header)
+{
+  if (profile == header->priv->profile)
+    header->priv->profile = NULL;
+}
+
 static void
 gtr_header_init (GtrHeader * header)
 {
   header->priv = GTR_HEADER_GET_PRIVATE (header);
 
   header->priv->nplurals = -1;
+  header->priv->profile = NULL;
+
+  header->priv->prof_manager = gtr_profile_manager_get_default ();
+
+  g_signal_connect (header->priv->prof_manager, "profile-removed",
+                    G_CALLBACK (profile_removed_cb), header);
 }
 
 static void
-gtr_header_finalize (GObject * object)
+gtr_header_dispose (GObject * object)
 {
-  G_OBJECT_CLASS (gtr_header_parent_class)->finalize (object);
+  GtrHeader *header = GTR_HEADER (object);
+
+  if (header->priv->prof_manager != NULL)
+    {
+      g_object_unref (header->priv->prof_manager);
+      header->priv->prof_manager = NULL;
+    }
+
+  if (header->priv->profile != NULL)
+    {
+      g_object_unref (header->priv->profile);
+      header->priv->profile = NULL;
+    }
+
+  G_OBJECT_CLASS (gtr_header_parent_class)->dispose (object);
 }
 
 static void
@@ -137,7 +179,7 @@ gtr_header_class_init (GtrHeaderClass * klass)
 
   g_type_class_add_private (klass, sizeof (GtrHeaderPrivate));
 
-  object_class->finalize = gtr_header_finalize;
+  object_class->dispose = gtr_header_dispose;
 }
 
 /* Public methods */
@@ -493,14 +535,17 @@ gtr_header_get_nplurals (GtrHeader * header)
 }
 
 static void
-set_profile_values (GtrHeader * header)
+set_profile_values (GtrHeader *header)
 {
-  if (gtr_prefs_manager_get_use_profile_values ())
+  GtrProfile *active_profile;
+
+  if (header->priv->profile != NULL)
+    active_profile = header->priv->profile;
+  else
+    active_profile = gtr_profile_manager_get_active_profile (header->priv->prof_manager);
+
+  if (gtr_prefs_manager_get_use_profile_values () && active_profile != NULL)
     {
-      GtrProfile *active_profile;
-
-      active_profile = gtr_application_get_active_profile (GTR_APP);
-
       gtr_header_set_translator (header,
                                  gtr_profile_get_author_name (active_profile),
                                  gtr_profile_get_author_email
@@ -513,7 +558,7 @@ set_profile_values (GtrHeader * header)
       gtr_header_set_encoding (header,
                                gtr_profile_get_encoding (active_profile));
       gtr_header_set_plural_forms (header,
-                                   gtr_profile_get_plurals (active_profile));
+                                   gtr_profile_get_plural_forms (active_profile));
     }
 }
 
@@ -538,8 +583,9 @@ update_po_date (GtrHeader * header)
 }
 
 static void
-update_comments (GtrHeader * header, const gchar * comments)
+update_comments (GtrHeader *header, const gchar *comments)
 {
+  GtrProfile *active_profile;
   GString *new_comments;
   GString *years;
   gchar **comment_lines;
@@ -548,15 +594,16 @@ update_comments (GtrHeader * header, const gchar * comments)
   gchar *current_year;
   gint i;
 
+  if (header->priv->profile != NULL)
+    active_profile = header->priv->profile;
+  else
+    active_profile = gtr_profile_manager_get_active_profile (header->priv->prof_manager);
+
   current_year = gtr_utils_get_current_year ();
 
   /* Save the previous translator to update the header's comment */
-  if (gtr_prefs_manager_get_use_profile_values ())
+  if (gtr_prefs_manager_get_use_profile_values () && active_profile != NULL)
     {
-      GtrProfile *active_profile;
-
-      active_profile = gtr_application_get_active_profile (GTR_APP);
-
       translator = g_strdup (gtr_profile_get_author_name (active_profile));
       email = g_strdup (gtr_profile_get_author_email (active_profile));
     }
@@ -655,3 +702,20 @@ gtr_header_update_header (GtrHeader * header)
   else
     add_default_comments (header);
 }
+
+void
+gtr_header_set_profile (GtrHeader  *header,
+                        GtrProfile *profile)
+{
+  g_return_if_fail (GTR_IS_HEADER (header));
+
+  header->priv->profile = g_object_ref (profile);
+}
+
+GtrProfile *
+gtr_header_get_profile (GtrHeader *header)
+{
+  g_return_val_if_fail (GTR_IS_HEADER (header), NULL);
+
+  return header->priv->profile;
+}
