@@ -37,6 +37,8 @@
 #include "gtr-statusbar.h"
 #include "gtr-utils.h"
 #include "gtr-window.h"
+#include "gtr-profile-manager.h"
+#include "gtr-status-combo-box.h"
 
 #include "egg-toolbars-model.h"
 #include "egg-toolbar-editor.h"
@@ -61,6 +63,8 @@
 #define GTR_STOCK_FUZZY_UNTRANS_NEXT "gtranslator-fuzzy-untranslated-next"
 #define GTR_STOCK_FUZZY_UNTRANS_PREV "gtranslator-fuzzy-untranslated-prev"
 
+#define PROFILE_DATA "GtrWidnowProfileData"
+
 #define GTR_WINDOW_GET_PRIVATE(object)	(G_TYPE_INSTANCE_GET_PRIVATE ( \
 					 (object),	\
 					 GTR_TYPE_WINDOW,     \
@@ -71,80 +75,84 @@ static void gtr_window_cmd_edit_toolbar (GtkAction * action,
 
 
 G_DEFINE_TYPE (GtrWindow, gtr_window, GTK_TYPE_WINDOW)
-     struct _GtrWindowPrivate
-     {
-       GtkWidget *main_box;
 
-       GtkWidget *menubar;
-       GtkWidget *view_menu;
-       GtkWidget *toolbar;
-       GtkActionGroup *always_sensitive_action_group;
-       GtkActionGroup *action_group;
-       GtkActionGroup *documents_list_action_group;
-       guint documents_list_menu_ui_id;
+struct _GtrWindowPrivate
+{
+  GtkWidget *main_box;
 
-       GtkWidget *notebook;
-       GtrTab *active_tab;
+  GtkWidget *menubar;
+  GtkWidget *view_menu;
+  GtkWidget *toolbar;
+  GtkActionGroup *always_sensitive_action_group;
+  GtkActionGroup *action_group;
+  GtkActionGroup *documents_list_action_group;
+  guint documents_list_menu_ui_id;
 
-       GtkWidget *dock;
-       GdlDockLayout *layout_manager;
-       GHashTable *widgets;
+  GtkWidget *notebook;
+  GtrTab *active_tab;
 
-       GtkWidget *statusbar;
+  GtkWidget *dock;
+  GdlDockLayout *layout_manager;
+  GHashTable *widgets;
 
-       GtkUIManager *ui_manager;
-       GtkRecentManager *recent_manager;
-       GtkWidget *recent_menu;
+  GtkWidget *statusbar;
 
-       GtkWidget *tm_menu;
+  GtkUIManager *ui_manager;
+  GtkRecentManager *recent_manager;
+  GtkWidget *recent_menu;
 
-       gint width;
-       gint height;
-       GdkWindowState window_state;
+  GtkWidget *tm_menu;
 
-       gboolean destroy_has_run:1;
-     };
+  gint width;
+  gint height;
+  GdkWindowState window_state;
 
-     enum
-     {
-       TARGET_URI_LIST = 100
-     };
+  GtrProfileManager *prof_manager;
+  GtkWidget *profile_combo;
 
-     static const GtkActionEntry always_sensitive_entries[] = {
+  gboolean destroy_has_run : 1;
+};
 
-       {"File", NULL, N_("_File")},
-       {"Edit", NULL, N_("_Edit")},
-       {"View", NULL, N_("_View")},
-       {"Search", NULL, N_("_Search")},
-       {"Go", NULL, N_("_Go")},
-       {"Documents", NULL, N_("_Documents")},
-       {"Help", NULL, N_("_Help")},
+enum
+{
+  TARGET_URI_LIST = 100
+};
 
-       /* File menu */
-       {"FileOpen", GTK_STOCK_OPEN, NULL, "<control>O",
-        N_("Open a PO file"),
-        G_CALLBACK (gtr_open_file_dialog)},
-       {"FileRecentFiles", NULL, N_("_Recent Files"), NULL,
-        NULL, NULL},
-       {"FileQuitWindow", GTK_STOCK_QUIT, NULL, "<control>Q",
-        N_("Quit the program"),
-        G_CALLBACK (gtr_file_quit)},
+static const GtkActionEntry always_sensitive_entries[] = {
 
-       /* Edit menu */
-       {"EditToolbar", NULL, N_("T_oolbar"), NULL, NULL,
-        G_CALLBACK (gtr_window_cmd_edit_toolbar)},
-       {"EditPreferences", GTK_STOCK_PREFERENCES, NULL, NULL,
-        N_("Edit gtr preferences"),
-        G_CALLBACK (gtr_actions_edit_preferences)},
-       {"EditHeader", GTK_STOCK_PROPERTIES, N_("_Header..."), NULL, NULL,
-        G_CALLBACK (gtr_actions_edit_header)},
+  {"File", NULL, N_("_File")},
+  {"Edit", NULL, N_("_Edit")},
+  {"View", NULL, N_("_View")},
+  {"Search", NULL, N_("_Search")},
+  {"Go", NULL, N_("_Go")},
+  {"Documents", NULL, N_("_Documents")},
+  {"Help", NULL, N_("_Help")},
 
-       /* Help menu */
-       {"HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1", NULL,
-        G_CALLBACK (gtr_cmd_help_contents)},
-       {"HelpAbout", GTK_STOCK_ABOUT, NULL, NULL, NULL,
-        G_CALLBACK (gtr_about_dialog)},
-     };
+  /* File menu */
+  {"FileOpen", GTK_STOCK_OPEN, NULL, "<control>O",
+   N_("Open a PO file"),
+   G_CALLBACK (gtr_open_file_dialog)},
+  {"FileRecentFiles", NULL, N_("_Recent Files"), NULL,
+   NULL, NULL},
+  {"FileQuitWindow", GTK_STOCK_QUIT, NULL, "<control>Q",
+   N_("Quit the program"),
+   G_CALLBACK (gtr_file_quit)},
+
+  /* Edit menu */
+  {"EditToolbar", NULL, N_("T_oolbar"), NULL, NULL,
+   G_CALLBACK (gtr_window_cmd_edit_toolbar)},
+  {"EditPreferences", GTK_STOCK_PREFERENCES, NULL, NULL,
+   N_("Edit gtranslator preferences"),
+   G_CALLBACK (gtr_actions_edit_preferences)},
+  {"EditHeader", GTK_STOCK_PROPERTIES, N_("_Header..."), NULL, NULL,
+   G_CALLBACK (gtr_actions_edit_header)},
+
+  /* Help menu */
+  {"HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1", NULL,
+   G_CALLBACK (gtr_cmd_help_contents)},
+  {"HelpAbout", GTK_STOCK_ABOUT, NULL, NULL, NULL,
+   G_CALLBACK (gtr_about_dialog)},
+};
 
 /* Normal items */
 static const GtkActionEntry entries[] = {
@@ -271,6 +279,10 @@ static const GtkActionEntry entries[] = {
    G_CALLBACK (gtr_actions_documents_next_document)}
 };
 
+static void          profile_combo_changed            (GtrStatusComboBox *combo,
+                                                       GtkMenuItem       *item,
+                                                       GtrWindow         *window);
+
 /*
  * Dock funcs
  */
@@ -312,6 +324,8 @@ static void
 on_layout_dirty_notify (GObject * object,
                         GParamSpec * pspec, GtrWindow * window)
 {
+  g_return_if_fail (GTR_IS_WINDOW (window));
+
   if (!strcmp (pspec->name, "dirty"))
     {
       gboolean dirty;
@@ -378,7 +392,7 @@ remove_from_widgets_hash (gpointer name,
 static void
 on_widget_destroy (GtkWidget * widget, GtrWindow * window)
 {
-  //DEBUG_PRINT ("Widget about to be destroyed");
+  DEBUG_PRINT ("Widget about to be destroyed");
   g_hash_table_foreach_remove (window->priv->widgets,
                                remove_from_widgets_hash, widget);
 }
@@ -403,7 +417,7 @@ on_widget_remove (GtkWidget * container,
   if (g_hash_table_foreach_remove (window->priv->widgets,
                                    remove_from_widgets_hash, widget))
     {
-      //DEBUG_PRINT ("Widget removed from container");
+      DEBUG_PRINT ("Widget removed from container");
     }
 }
 
@@ -414,7 +428,7 @@ on_widget_removed_from_hash (gpointer widget)
   GtkWidget *menuitem;
   GdlDockItem *dockitem;
 
-  //DEBUG_PRINT ("Removing widget from hash");
+  DEBUG_PRINT ("Removing widget from hash");
 
   window = g_object_get_data (G_OBJECT (widget), "window-object");
   dockitem = g_object_get_data (G_OBJECT (widget), "dockitem");
@@ -931,21 +945,27 @@ set_window_title (GtrWindow * window, gboolean with_path)
 
   if (with_path)
     {
-      po = gtr_tab_get_po (GTR_TAB (gtr_window_get_active_tab (window)));
+      gchar *path;
+
       active_tab = gtr_window_get_active_tab (window);
+      po = gtr_tab_get_po (active_tab);
       state = gtr_po_get_state (gtr_tab_get_po (active_tab));
       po = gtr_tab_get_po (active_tab);
       file = gtr_po_get_location (po);
+      path = g_file_get_path (file);
 
       if (state == GTR_PO_STATE_MODIFIED)
-        title = g_strdup_printf (_("gtr - *%s"), g_file_get_path (file));
+        /* Translators: this is the title of the window with a modified document */
+        title = g_strdup_printf (_("*%s - gtranslator"), path);
       else
-        title = g_strdup_printf (_("gtr - %s"), g_file_get_path (file));
+        /* Translators: this is the title of the window with a document opened */
+        title = g_strdup_printf (_("%s - gtranslator"), path);
 
+      g_free (path);
       g_object_unref (file);
     }
   else
-    title = g_strdup (_("gtr"));
+    title = g_strdup (_("gtranslator"));
 
   gtk_window_set_title (GTK_WINDOW (window), title);
   g_free (title);
@@ -960,9 +980,12 @@ notebook_switch_page (GtkNotebook * nb,
   GList *msg;
   GtrView *view;
   GtrPo *po;
+  GtrHeader *header;
+  GtrProfile *profile;
   gint n_pages;
   GtkAction *action;
   gchar *action_name;
+  GList *profile_items, *l;
 
   tab = GTR_TAB (gtk_notebook_get_nth_page (nb, page_num));
   if (tab == window->priv->active_tab)
@@ -993,6 +1016,34 @@ notebook_switch_page (GtkNotebook * nb,
   msg = gtr_po_get_current_message (po);
   gtr_window_update_statusbar_message_count (tab, msg->data, window);
 
+  header = gtr_po_get_header (po);
+  profile = gtr_header_get_profile (header);
+
+  if (profile == NULL)
+    profile = gtr_profile_manager_get_active_profile (window->priv->prof_manager);
+
+  profile_items = gtr_status_combo_box_get_items (GTR_STATUS_COMBO_BOX (window->priv->profile_combo));
+
+  for (l = profile_items; l != NULL; l = g_list_next (l))
+    {
+      GtrProfile *item_profile;
+
+      item_profile = GTR_PROFILE (g_object_get_data (G_OBJECT (l->data),
+                                                     PROFILE_DATA));
+
+      if (item_profile == profile)
+        {
+          g_signal_handlers_block_by_func (window->priv->profile_combo,
+                                           profile_combo_changed,
+                                           window);
+          gtr_status_combo_box_set_item (GTR_STATUS_COMBO_BOX (window->priv->profile_combo),
+                                         GTK_MENU_ITEM (l->data));
+          g_signal_handlers_unblock_by_func (window->priv->profile_combo,
+                                             profile_combo_changed,
+                                             window);
+        }
+    }
+
   /* activate the right item in the documents menu */
   action_name = g_strdup_printf ("Tab_%d", page_num);
   action =
@@ -1017,15 +1068,17 @@ notebook_page_removed (GtkNotebook * notebook,
 {
   gint n_pages;
 
-  /*
-   * Set the window title
-   */
+  /* Set the window title */
   n_pages = gtk_notebook_get_n_pages (notebook);
   if (n_pages == 1)
     set_window_title (window, TRUE);
   else
     set_window_title (window, FALSE);
 
+  /* Hide the profile combo */
+  if (n_pages == 0)
+    gtk_widget_hide (window->priv->profile_combo);
+
   update_documents_list_menu (window);
 }
 
@@ -1116,15 +1169,16 @@ notebook_tab_added (GtkNotebook * notebook,
 
   g_return_if_fail (GTR_IS_TAB (tab));
 
-  /*
-   * Set the window title
-   */
+  /* Set the window title */
   n_pages = gtk_notebook_get_n_pages (notebook);
   if (n_pages == 1)
     set_window_title (window, TRUE);
   else
     set_window_title (window, FALSE);
 
+  /* Show the profile combo */
+  gtk_widget_show (window->priv->profile_combo);
+
   views = gtr_tab_get_all_views (tab, FALSE, TRUE);
 
   while (views)
@@ -1158,8 +1212,8 @@ notebook_tab_added (GtkNotebook * notebook,
 
   update_documents_list_menu (window);
 
-  gtr_plugins_engine_update_plugins_ui
-    (gtr_plugins_engine_get_default (), window, FALSE);
+  gtr_plugins_engine_update_plugins_ui (gtr_plugins_engine_get_default (),
+                                        window, FALSE);
 }
 
 void
@@ -1317,17 +1371,16 @@ static void
 menu_item_select_cb (GtkMenuItem * proxy, GtrWindow * window)
 {
   GtkAction *action;
-  char *message;
+  const gchar *message;
 
   action = gtk_activatable_get_related_action (GTK_ACTIVATABLE (proxy));
   g_return_if_fail (action != NULL);
 
-  g_object_get (G_OBJECT (action), "tooltip", &message, NULL);
+  message = gtk_action_get_tooltip (action);
   if (message)
     {
-      gtr_statusbar_push_default (GTR_STATUSBAR
-                                  (window->priv->statusbar), message);
-      g_free (message);
+      gtr_statusbar_push_default (GTR_STATUSBAR (window->priv->statusbar),
+                                  message);
     }
 }
 
@@ -1364,6 +1417,208 @@ disconnect_proxy_cb (GtkUIManager * manager,
     }
 }
 
+static void
+profile_combo_changed (GtrStatusComboBox *combo,
+                       GtkMenuItem       *item,
+                       GtrWindow         *window)
+{
+  GtrTab *tab;
+  GtrPo *po;
+  GtrHeader *header;
+  GtrProfile *profile;
+
+  tab = gtr_window_get_active_tab (window);
+
+  if (tab == NULL)
+    return;
+
+  po = gtr_tab_get_po (tab);
+  header = gtr_po_get_header (po);
+
+  profile = GTR_PROFILE (g_object_get_data (G_OBJECT (item), PROFILE_DATA));
+
+  gtr_header_set_profile (header, profile);
+}
+
+static void
+fill_profile_combo (GtrWindow *window)
+{
+  GSList *profiles, *l;
+  GtkWidget *menu_item;
+  const gchar *name;
+
+  profiles = gtr_profile_manager_get_profiles (window->priv->prof_manager);
+
+  for (l = profiles; l != NULL; l = g_slist_next (l))
+    {
+      GtrProfile *profile = GTR_PROFILE (l->data);
+
+      name = gtr_profile_get_name (profile);
+      menu_item = gtk_menu_item_new_with_label (name);
+      gtk_widget_show (menu_item);
+
+      g_object_set_data (G_OBJECT (menu_item), PROFILE_DATA, profile);
+      gtr_status_combo_box_add_item (GTR_STATUS_COMBO_BOX (window->priv->profile_combo),
+                                     GTK_MENU_ITEM (menu_item),
+                                     name);
+    }
+
+  if (profiles == NULL)
+    {
+      name = _("No profile");
+
+      menu_item = gtk_menu_item_new_with_label (name);
+
+      g_object_set_data (G_OBJECT (menu_item), PROFILE_DATA, NULL);
+      gtr_status_combo_box_add_item (GTR_STATUS_COMBO_BOX (window->priv->profile_combo),
+                                     GTK_MENU_ITEM (menu_item),
+                                     name);
+    }
+}
+
+static void
+create_statusbar (GtrWindow *window,
+                  GtkWidget *box)
+{
+  window->priv->statusbar = gtr_statusbar_new ();
+
+  gtk_box_pack_end (GTK_BOX (box), window->priv->statusbar, TRUE, TRUE, 0);
+
+  gtk_widget_show (window->priv->statusbar);
+
+  window->priv->profile_combo = gtr_status_combo_box_new (_("Profile"));
+  gtk_box_pack_start (GTK_BOX (window->priv->statusbar),
+                      window->priv->profile_combo, FALSE, TRUE, 0);
+
+  g_signal_connect (window->priv->profile_combo, "changed",
+                    G_CALLBACK (profile_combo_changed), window);
+
+  fill_profile_combo (window);
+}
+
+static void
+on_active_profile_changed (GtrProfileManager *manager,
+                           GtrProfile        *profile,
+                           GtrWindow         *window)
+{
+  GList *items, *l;
+  GtrTab *tab;
+  GtrPo *po;
+  GtrHeader *header;
+
+  tab = gtr_window_get_active_tab (window);
+
+  if (tab == NULL)
+    return;
+
+  po = gtr_tab_get_po (tab);
+  header = gtr_po_get_header (po);
+
+  items = gtr_status_combo_box_get_items (GTR_STATUS_COMBO_BOX (window->priv->profile_combo));
+
+  for (l = items; l != NULL; l = g_list_next (l))
+    {
+      GtkMenuItem *menu_item;
+      GtrProfile *item_profile;
+
+      menu_item = GTK_MENU_ITEM (l->data);
+
+      item_profile = GTR_PROFILE (g_object_get_data (G_OBJECT (menu_item),
+                                  PROFILE_DATA));
+
+      if (item_profile == profile && gtr_header_get_profile (header) == NULL)
+        {
+          g_signal_handlers_block_by_func (window->priv->profile_combo,
+                                           profile_combo_changed, window);
+          gtr_status_combo_box_set_item (GTR_STATUS_COMBO_BOX (window->priv->profile_combo),
+                                         menu_item);
+          g_signal_handlers_unblock_by_func (window->priv->profile_combo,
+                                             profile_combo_changed, window);
+        }
+    }
+}
+
+static void
+on_profile_added (GtrProfileManager *manager,
+                  GtrProfile        *profile,
+                  GtrWindow         *window)
+{
+  GtkMenuItem *menu_item;
+  GList *items;
+
+  /* check that the item is not a "No profile" item */
+  items = gtr_status_combo_box_get_items (GTR_STATUS_COMBO_BOX (window->priv->profile_combo));
+
+  if (items->next == NULL &&
+      (g_object_get_data (G_OBJECT (items->data), PROFILE_DATA) == NULL))
+    {
+      menu_item = GTK_MENU_ITEM (items->data);
+
+      gtk_menu_item_set_label (menu_item,
+                               gtr_profile_get_name (profile));
+
+      g_object_set_data (G_OBJECT (menu_item), PROFILE_DATA, profile);
+    }
+  else
+    {
+      const gchar *name;
+
+      name = gtr_profile_get_name (profile);
+      menu_item = GTK_MENU_ITEM (gtk_menu_item_new_with_label (name));
+      gtk_widget_show (GTK_WIDGET (menu_item));
+
+      g_object_set_data (G_OBJECT (menu_item), PROFILE_DATA, profile);
+      gtr_status_combo_box_add_item (GTR_STATUS_COMBO_BOX (window->priv->profile_combo),
+                                     menu_item, name);
+    }
+}
+
+static void
+on_profile_removed (GtrProfileManager *manager,
+                    GtrProfile        *profile,
+                    GtrWindow         *window)
+{
+  GList *items, *l;
+
+  items = gtr_status_combo_box_get_items (GTR_STATUS_COMBO_BOX (window->priv->profile_combo));
+
+  for (l = items; l != NULL; l = g_list_next (l))
+    {
+      GtrProfile *prof;
+
+      prof = GTR_PROFILE (g_object_get_data (G_OBJECT (l->data), PROFILE_DATA));
+      if (prof == profile)
+        gtr_status_combo_box_remove_item (GTR_STATUS_COMBO_BOX (window->priv->profile_combo),
+                                          GTK_MENU_ITEM (l->data));
+    }
+}
+
+static void
+on_profile_modified (GtrProfileManager *manager,
+                     GtrProfile        *old_profile,
+                     GtrProfile        *new_profile,
+                     GtrWindow         *window)
+{
+  GList *items, *l;
+
+  items = gtr_status_combo_box_get_items (GTR_STATUS_COMBO_BOX (window->priv->profile_combo));
+
+  for (l = items; l != NULL; l = g_list_next (l))
+    {
+      GtrProfile *profile;
+
+      profile = GTR_PROFILE (g_object_get_data (G_OBJECT (l->data), PROFILE_DATA));
+      if (profile == old_profile)
+        {
+          gtk_menu_item_set_label (GTK_MENU_ITEM (l->data),
+                                   gtr_profile_get_name (new_profile));
+          g_object_set_data (G_OBJECT (l->data), PROFILE_DATA, new_profile);
+
+          return;
+        }
+    }
+}
+
 static void
 gtr_window_draw (GtrWindow * window)
 {
@@ -1508,11 +1763,7 @@ gtr_window_draw (GtrWindow * window)
   gtk_widget_show (hbox);
 
   /* statusbar & progress bar */
-  window->priv->statusbar = gtr_statusbar_new ();
-
-  gtk_box_pack_end (GTK_BOX (hbox), window->priv->statusbar, TRUE, TRUE, 0);
-
-  gtk_widget_show (priv->statusbar);
+  create_statusbar (window, hbox);
 }
 
 static void
@@ -1527,6 +1778,19 @@ gtr_window_init (GtrWindow * window)
 
   window->priv->destroy_has_run = FALSE;
 
+  /* Profile manager */
+  window->priv->prof_manager = gtr_profile_manager_get_default ();
+
+  g_signal_connect (window->priv->prof_manager, "active-profile-changed",
+                    G_CALLBACK (on_active_profile_changed), window);
+  g_signal_connect (window->priv->prof_manager, "profile-added",
+                    G_CALLBACK (on_profile_added), window);
+  g_signal_connect (window->priv->prof_manager, "profile-removed",
+                    G_CALLBACK (on_profile_removed), window);
+  g_signal_connect (window->priv->prof_manager, "profile-modified",
+                    G_CALLBACK (on_profile_modified), window);
+
+  /* set up the window widgets */
   gtr_window_draw (window);
 
   set_sensitive_according_to_window (window);
@@ -1588,6 +1852,8 @@ gtr_window_dispose (GObject * object)
   GtrWindow *window = GTR_WINDOW (object);
   GtrWindowPrivate *priv = window->priv;
 
+  DEBUG_PRINT ("window dispose");
+
   if (priv->ui_manager)
     {
       g_object_unref (priv->ui_manager);
@@ -1599,6 +1865,17 @@ gtr_window_dispose (GObject * object)
       priv->action_group = NULL;
     }
 
+  if (priv->prof_manager != NULL)
+    {
+      g_object_unref (priv->prof_manager);
+      priv->prof_manager = NULL;
+    }
+
+  /* Now that there have broken some reference loops,
+   * force collection again.
+   */
+  gtr_plugins_engine_garbage_collect (gtr_plugins_engine_get_default ());
+
   G_OBJECT_CLASS (gtr_window_parent_class)->dispose (object);
 }
 
@@ -1636,9 +1913,23 @@ gtr_window_destroy (GtkObject * object)
 
   window = GTR_WINDOW (object);
 
+  DEBUG_PRINT ("Destroy window");
+
   if (!window->priv->destroy_has_run)
     {
       save_panes_state (window);
+
+      if (window->priv->widgets)
+        {
+          g_hash_table_destroy (window->priv->widgets);
+          window->priv->widgets = NULL;
+        }
+
+      if (window->priv->layout_manager)
+        {
+          g_object_unref (window->priv->layout_manager);
+          window->priv->layout_manager = NULL;
+        }
       window->priv->destroy_has_run = TRUE;
     }
 
